package com.engidea.winjcom;

import com.engidea.winjcom.exception.PortInUseException;
import com.engidea.winjcom.exception.UnsupportedCommOperationException;

import java.io.*;
import java.util.*;

/**
  WinJcom is a native interface to serial ports in java.
  Copyright 2007 by Damiano Bolla, Jcomm@engidea.com

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Library General Public
  License as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.
  This can be used with commercial products and you are not obliged 
  to share your work with anybody.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.

  You should have received a copy of the GNU Library General Public
  License along with this library; if not, write to the Free
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/**
 * Serial deveices get loaded starting from this one !
 * Basically you can just 
 * aPort = CommPortIdentifier.getPortIdentifier()
 * aPort.open (aPort.getName(), aPort.getType())
 * If you are wondering WHY this round... well, yes, this API is a mess...
 * This is the reason the portOwner handling is not implemented, it does not really make sense for devices
 * that are managed by the OS, the OS KNOWS who the owner is.
 * If an APPLICATION wants to manage its ports, then let it do, but not a driver !
 * ISSUE: Ports can be removed from the system, but there is no remove method in the API
 */
public class CommPortIdentifier
  {
  public static final int PORT_SERIAL = 1;        // as from javadoc
  public static final int PORT_PARALLEL = 2;      // as from javadoc
  
  private static boolean    debug = false;
  private static JcomDriver winjserDriver;
  private static Object     syncObj;
  private static ArrayList  identifierList;
  
  private CommDriver portDriver;   // the driver that can open this port.
  private String     portName;     // the name of the port
  private int        portType;     // what type of port it is

  static 
    {
    if (debug) System.out.println("CommPortIdentifier:static initialization()");

    try
      {
      syncObj = new Object();
      identifierList = new ArrayList();

      winjserDriver = new JcomDriver(debug);
      winjserDriver.initialize();
      } 
    catch (Throwable e)
      {
      System.err.println(e + " thrown while loading ");
      }
    }


  /**
   * THis shouldnot be public, after all you can only add a port ONLY if you have the driver !
   * @param portName
   * @param type
   * @param portDriver
   */
  static void addPortName(String portName, int type, CommDriver portDriver)
    {
    // if the port I try to add is already present, just skip.
    if ( getPortIdentifier(portName) != null ) return;
    
    synchronized (syncObj)
      {
      identifierList.add(new CommPortIdentifier(portName, type, portDriver) );
      }
    }

  /**
   * Used by the driver whan it wants to rescan the ports
   */
  static void clearPortList ()
    {
    synchronized (syncObj)
      {
      identifierList.clear();
      }
    }

  /**
   * Note that can return null if you give a null as parameter.
   * It should have been possible to give the type, what if two ports share the same name but different type ?!
   * @param portName
   * @return a n identifier or null if it cannot find it.
   */
  public static CommPortIdentifier getPortIdentifier(String portName)
    {
    if ( portName == null ) return null;

    synchronized (syncObj)
      {
      for (int index=0; index<identifierList.size(); index++)
        {
        CommPortIdentifier risul = (CommPortIdentifier)identifierList.get(index);

        if ( portName.equals(risul.portName) ) return risul;
        }
      }

    return null;
    }


  /**
   * This is the one I normally use.
   * NOTE that the type returned is different thatn standard API....
   */
  public static Iterator getPortIdentifiers()
    {
    winjserDriver.scanStandardPorts();

    return identifierList.iterator();
    }

  // ================================ END OF STATIC =============================
  
  /**
   * Constructor 
   */
  private CommPortIdentifier(String portName, int portType, CommDriver portDriver)
    {
    this.portName   = portName;
    this.portType   = portType;
    this.portDriver = portDriver;
    }


  /**
   * Open this identifier and returns a ComPort (that can be a SerialPort)
   * @param portOwner NOT USED
   * @param portOpenTimeout NOT USED
   * @return
   * @throws com.engidea.winjcom.exception.PortInUseException NOT THROWN
   * @throws IOException can be thrown if open fails 
   */
  public synchronized CommPort open(String portOwner, int portOpenTimeout) 
    throws PortInUseException, IOException
    {    
    CommPort aPort = portDriver.getCommPort(portName,portType);
    
    aPort.open();
    
    return aPort;
    }

  public String getName()
    {
    return portName;
    }
  
  public int getPortType()
    {
    return portType;
    }


  public static CommPortIdentifier getPortIdentifier(CommPort commPort) 
    {
    throw new IllegalStateException("Not Implemented");
    }

  public String getCurrentOwner()
    {
    throw new IllegalStateException("Not Implemented");
    }

  public synchronized boolean isCurrentlyOwned()
    {
    throw new IllegalStateException("Not Implemented");
    }

  public void addPortOwnershipListener(CommPortOwnershipListener c)
    {
    throw new IllegalStateException("Not implemented");
    }

  public synchronized CommPort open(FileDescriptor f) throws UnsupportedCommOperationException
    {
    throw new UnsupportedCommOperationException("Not implemented");
    }

  public void removePortOwnershipListener(CommPortOwnershipListener c)
    {
    throw new IllegalStateException("Not implemented");
    }

  void fireOwnershipEvent(int eventType)
    {
    throw new IllegalStateException("Not implemented");
    }
  }

