package com.engidea.winjcom;

import com.engidea.winjcom.exception.PortInUseException;

import java.io.IOException;

/**
  WinJcom is a native interface to serial ports in java.
  Copyright 2007 by Damiano Bolla, Jcomm@engidea.com

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Library General Public
  License as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.
  This can be used with commercial products and you are not obliged 
  to share your work with anybody.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.

  You should have received a copy of the GNU Library General Public
  License along with this library; if not, write to the Free
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/



/**
 * this is attached with the CommPortIdentifier, gets loaded at startup.
 */
public class JcomDriver implements CommDriver
  {
  private static final String classname="JcomDriver.";

  public static final String  jcomNativeLibrary="winjcom";
  
  private boolean debug;
  
  /**
   * The constructor is package protected since it should NOT be possibile to get a driver
   * WITHOUT going trough the COmmPortIdentifier !!!
   */
  JcomDriver (boolean debug)
    {
    this.debug = debug;
    }
    
  private void report ( String msg )
    {
    System.out.println(msg);
    }

  private native void nativeInitialize(int debugMaks);

  /**
   * Initialize the native library.
   * This also starts a thread that will scan available ports.
   */
  public void initialize()
    {
    if (debug) report (classname+"initialize native lib=" + jcomNativeLibrary);

    // load native library
    System.loadLibrary(jcomNativeLibrary);
    
    // Init the native part of the library
    // the parameter is the debug mask, look at the source for the possible values
    nativeInitialize(0x0);
    }

  // Note that his one does NOT leave the port "open"
  // it throws exception to signal what is happening wrong.
  private native void nativeTestPort(String dev, int type) throws IOException;

  private void testAndRegisterPort(String portName )
    {
    try
      {
      // this will throw exception if this port is not available.
      nativeTestPort(portName, CommPortIdentifier.PORT_SERIAL);
      
      if ( debug ) report (classname+"add port="+portName);
      
      // register this port as using this driver
        CommPortIdentifier.addPortName(portName, CommPortIdentifier.PORT_SERIAL, this);
      }
    catch ( Exception exc )
      {
      if ( debug ) report (classname+"add port="+portName+"exc="+exc);
      }
    }

  /**
   * You may ask to scan standard ports at any time, to see if any new port is available
   * or to clean up possible old ports.
   */
  public void scanStandardPorts ()
    {
    CommPortIdentifier.clearPortList();

    testAndRegisterPort("COM1");
    testAndRegisterPort("COM2");
    testAndRegisterPort("COM3");
    testAndRegisterPort("COM4");
    testAndRegisterPort("COM5");      
    testAndRegisterPort("COM6");      
    testAndRegisterPort("COM7");      
    testAndRegisterPort("COM8");      
    testAndRegisterPort("COM9");      
    testAndRegisterPort("COM10");      
    testAndRegisterPort("COM11");      
    testAndRegisterPort("COM12");      
    testAndRegisterPort("COM13");      
    testAndRegisterPort("COM14");      
    testAndRegisterPort("COM15");      
    }

  


  /**
   * This is how you get a comm port.
   * NOTE: The port is NOT opened !!!
   * @param PortName
   * @param PortType
   * @return
   */
  public CommPort getCommPort(String PortName, int PortType) throws PortInUseException
    {
    if (debug)
      System.out.println(classname+"getCommPort(" + PortName + "," + PortType + ")");

    if ( PortType == CommPortIdentifier.PORT_SERIAL )
      return new JcomPort(PortName);

    return null;
    }
    
  }
